@@ -11,7 +11,6 @@ __author__ = 'Martin Pitt'
 __copyright__ = '(c) 2012 Canonical Ltd.'
 
 import copy
-import imp
 import importlib
 import os
 import sys
@@ -44,7 +43,9 @@ def load_module(name: str):
     '''Load a mock template Python module from dbusmock/templates/'''
 
     if os.path.exists(name) and os.path.splitext(name)[1] == '.py':
-        mod = imp.new_module(os.path.splitext(os.path.basename(name))[0])
+        spec = importlib.util.spec_from_file_location(os.path.splitext(os.path.basename(name))[0], name)
+        assert spec
+        mod = importlib.util.module_from_spec(spec)
         with open(name) as f:
             exec(f.read(), mod.__dict__, mod.__dict__)  # pylint: disable=exec-used
         return mod
@@ -88,6 +89,36 @@ def _format_args(args):
     return s
 
 
+def _wrap_in_dbus_variant(value):
+    dbus_types = [
+        dbus.types.Array,
+        dbus.types.ByteArray,
+        dbus.types.Int16,
+        dbus.types.ObjectPath,
+        dbus.types.Struct,
+        dbus.types.UInt64,
+        dbus.types.Boolean,
+        dbus.types.Dictionary,
+        dbus.types.Int32,
+        dbus.types.Signature,
+        dbus.types.UInt16,
+        dbus.types.UnixFd,
+        dbus.types.Byte,
+        dbus.types.Double,
+        dbus.types.Int64,
+        dbus.types.String,
+        dbus.types.UInt32,
+    ]
+    if isinstance(value, dbus.String):
+        return dbus.String(str(value), variant_level=1)
+    if type(value) in dbus_types:
+        return type(value)(value.conjugate(), variant_level=1)
+    if isinstance(value, str):
+        return dbus.String(value, variant_level=1)
+    raise dbus.exceptions.DBusException(
+        'could not wrap type {}'.format(type(value)))
+
+
 class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-attributes
     '''Mock D-Bus object
 
@@ -128,6 +159,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         self._template: Optional[str] = None
         self._template_parameters: Optional[PropsType] = None
 
+        # pylint: disable=consider-using-with
         self.logfile = open(logfile, 'wb') if logfile else None
         self.is_logfile_owner = True
         self.call_log: List[CallLogType] = []
@@ -400,6 +432,42 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         for method in methods:
             self.AddMethod(interface, *method)
 
+    def _set_property(self, interface, name, value):
+        # copy.copy removes one level of variant-ness, which means that the
+        # types get exported in introspection data correctly, but we can't do
+        # this for container types.
+        if not isinstance(value, (dbus.Dictionary, dbus.Array)):
+            value = copy.copy(value)
+
+        self.props.setdefault(interface, {})[name] = value
+
+    @dbus.service.method(MOCK_IFACE,
+                         in_signature='sa{sv}',
+                         out_signature='')
+    def UpdateProperties(self, interface: str, properties: PropsType) -> None:
+        '''Update properties on this object and send a PropertiesChanged signal
+
+        interface: D-Bus interface to update this to. For convenience you can
+                   specify '' here to add the property to the object's main
+                   interface (as specified on construction).
+        properties: A property_name (string) → value map
+        '''
+        changed_props = {}
+
+        for name, value in properties.items():
+            if not interface:
+                interface = self.interface
+            if name not in self.props.get(interface, {}):
+                raise dbus.exceptions.DBusException(
+                    'property %s not found' % name,
+                    name=interface + '.NoSuchProperty')
+
+            self._set_property(interface, name, value)
+            changed_props[name] = _wrap_in_dbus_variant(value)
+
+        self.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
+            interface, changed_props, []])
+
     @dbus.service.method(MOCK_IFACE,
                          in_signature='ssv',
                          out_signature='')
@@ -419,13 +487,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                 'property %s already exists' % name,
                 name=self.interface + '.PropertyExists')
 
-        # copy.copy removes one level of variant-ness, which means that the
-        # types get exported in introspection data correctly, but we can't do
-        # this for container types.
-        if not isinstance(value, (dbus.Dictionary, dbus.Array)):
-            value = copy.copy(value)
-
-        self.props.setdefault(interface, {})[name] = value
+        self._set_property(interface, name, value)
 
     @dbus.service.method(MOCK_IFACE,
                          in_signature='sa{sv}',
@@ -652,15 +714,15 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
 
         mock_interfaces = orig_interfaces.copy()
         for iface, methods in self.methods.items():
-            for method in methods:
-                mock_interfaces.setdefault(iface, {})[method] = self.methods[iface][method][3]
+            for method, impl in methods.items():
+                mock_interfaces.setdefault(iface, {})[method] = impl[3]
         self._dbus_class_table[cls] = mock_interfaces
 
         xml = dbus.service.Object.Introspect(self, object_path, connection)
 
         tree = ElementTree.fromstring(xml)
 
-        for name in self.props:
+        for name, name_props in self.props.items():
             # We might have properties for new interfaces we don't know about
             # yet. Try to find an existing <interface> node named after our
             # interface to append to, and create one if we can't.
@@ -669,7 +731,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                 interface = ElementTree.Element("interface", {"name": name})
                 tree.append(interface)
 
-            for prop, val in self.props[name].items():
+            for prop, val in name_props.items():
                 if val is None:
                     # can't guess type from None, skip
                     continue
